@@ -3,7 +3,7 @@
 ;; Copyright (C) 2006-2009, 2011-2012, 2015-2018
 ;;   Phil Hagelberg, Doug Alcorn, Will Farrington, Chen Bin
 ;;
-;; Version: 6.0.1
+;; Version: 6.0.7
 ;; Author: Phil Hagelberg, Doug Alcorn, and Will Farrington
 ;; Maintainer: Chen Bin <chenbin.sh@gmail.com>
 ;; URL: https://github.com/technomancy/find-file-in-project
@@ -38,10 +38,7 @@
 ;; - Works on Windows with minimum setup
 ;; - Works on Tramp Mode (https://www.emacswiki.org/emacs/TrampMode)
 ;; - fd (faster alternative of find, see https://github.com/sharkdp/fd) is supported
-;; - Uses native API `completing-read' and supports ido/helm/ivy/consult/selectrum out of box.
-;;
-;;   Ido setup,
-;;     (ido-mode 1)
+;; - Uses native API `completing-read' and supports helm/ivy/consult/selectrum out of box.
 ;;
 ;;   Helm setup,
 ;;     (helm-mode 1)
@@ -49,6 +46,9 @@
 ;;   Ivy setup,
 ;;     (ivy-mode 1)
 ;;
+;;   Ido setup,
+;;     (setq ffip-prefer-ido-mode t)
+;;
 ;; Usage,
 ;;   - You can insert "(setq ffip-use-rust-fd t)" into ".emacs" to use fd (alternative of find)
 ;;   - `find-file-in-project-at-point' guess the file path at point and
@@ -157,6 +157,7 @@
 (require 'diff-mode)
 (require 'windmove)
 (require 'subr-x)
+(require 'ido)
 
 (defgroup ffip nil
   "Find File in Project."
@@ -177,6 +178,11 @@
   :type 'boolean
   :safe #'booleanp)
 
+(defcustom ffip-prefer-ido-mode nil
+  "Prefer `ido-completing-read' to filter file candidates."
+  :group 'ffip
+  :type 'boolean
+  :safe #'booleanp)
 
 (defcustom ffip-rust-fd-extra-opts ""
   "Rust fd extra options passed to cli."
@@ -525,21 +531,23 @@ If CHECK-ONLY is true, only do the check."
 (defun ffip--win-executable-find (exe)
   "Find EXE on windows."
   (let* ((drivers '("c" "d" "e" "g" "h" "i" "j" "k"))
-          (i 0)
-          j
-          (dirs '(":\\\\cygwin64\\\\bin\\\\"
+         (i 0)
+         j
+         (dirs '(":\\\\cygwin64\\\\bin\\\\"
                  ":\\\\msys64\\\\usr\\\\bin\\\\"))
-          rlt)
-     (while (and (not rlt)
-                 (< i (length dirs)))
-       (setq j 0)
-       (while (and (not rlt)
-                   (< j (length drivers)))
-         (setq rlt (executable-find (concat (nth j drivers) (nth i dirs) exe)))))
-     (unless rlt
-       ;; nothing found, fall back to exe
-       (setq rlt exe))
-     rlt))
+         rlt)
+    (while (and (not rlt)
+                (< i (length dirs)))
+      (setq j 0)
+      (while (and (not rlt)
+                  (< j (length drivers)))
+        (setq rlt (executable-find (concat (nth j drivers) (nth i dirs) exe)))
+        (setq j (1+ j)))
+      (setq i (1+ i)))
+    (unless rlt
+      ;; nothing found, fall back to exe
+      (setq rlt exe))
+    rlt))
 
 (defun ffip--executable-find ()
   "Find EXE on all environments."
@@ -626,6 +634,10 @@ This function returns the selected candidate or nil."
       ;; select the only candidate immediately
       (setq selected (car collection)))
 
+     (ffip-prefer-ido-mode
+      ;; ido can only handle list of strings
+      (setq selected (ido-completing-read prompt (mapcar 'car collection))))
+
      (t
       (setq selected (completing-read prompt collection))
       (setq selected (or (assoc selected collection) selected))))
@@ -1017,8 +1029,11 @@ Set `ffip-find-relative-path-callback' to format the result,
           ;; only one item in project files
           (if ,find-directory-p
               (setq file (file-name-as-directory file)))
-          (setq file (file-relative-name file (file-name-directory buffer-file-name)))
-          (funcall ffip-find-relative-path-callback file))))
+              (setq file (file-relative-name file
+                                             (if buffer-file-name
+                                                 (file-name-directory buffer-file-name)
+                                               (expand-file-name default-directory))))
+              (funcall ffip-find-relative-path-callback file))))
      (t
       (message "Nothing found!")))))
 
@@ -1296,33 +1311,37 @@ If REVERSE is t, applied patch is reverted."
 If ABSOLUTE-PATH-P is t, old path is replaced by correct absolute path.
 Or else it's replaced by relative path."
   (interactive "P")
-  (let* ((fn (thing-at-point 'filename))
+  (let* ((filename (ffip-guess-file-name-at-point))
          full-path
          cands)
     (cond
-     ((not fn)
+     ((not filename)
       (message "There is no file path at point."))
 
      ;; path at point is a path of physical file
-     ((setq full-path (ffip--guess-physical-path fn))
+     ((setq full-path (ffip--guess-physical-path filename))
       nil)
 
      ;; find a file
-     ((setq cands (ffip-project-search (replace-regexp-in-string ffip-relative-path-pattern "" fn)))
+     ((setq cands (ffip-project-search (replace-regexp-in-string ffip-relative-path-pattern "" filename)))
       (cond
        ((eq (length cands) 1)
         (setq full-path (nth 0 cands)))
        (t
-        (ffip-completing-read "Find file: "
-                              cands
-                              `(lambda (file)
-                                 (setq ,full-path file)))))))
+        (setq full-path (completing-read "Find file: " cands))))))
 
     (when full-path
       (if (consp full-path) (setq full-path (cdr full-path)))
+
+      ;; convert to actual full path
+      (let* ((default-directory (ffip-get-project-root-directory)))
+        (setq full-path (file-truename full-path)))
+
       (let* ((bounds (bounds-of-thing-at-point 'filename))
              (path (if absolute-path-p full-path
-                     (file-relative-name full-path))))
+                     (file-relative-name full-path
+                                         ;; compare with current file's directory if possible
+                                         (and buffer-file-name (file-name-directory buffer-file-name))))))
         (goto-char (car bounds))
         (delete-region (car bounds) (cdr bounds))
         (insert (replace-regexp-in-string "/index\\.[jt]s" "" path))))))
